
// mon.c  monitor for command line interpreter
//    						13 Nov 2017

#include "npu.h"

#define PRIVATE     static

#define MXBUF  80
#define sep  "\t\n "

#define eqs(s1,s2)  (strcmp(s1,s2) == 0)

// definition of registers and memory
extern int R[][32], LDS[], PC, IR, RNG, M[];
extern int icnt, runflag, stopf;	// control

int brkp = -1;						// breakpoint

char ibuf[MXBUF], *cp = NULL;		// input buffer
FILE *fi, *fo;
int display = 1;					// display control

PRIVATE char *tok(void){			// get one token from input
	if( cp != NULL ) cp = strtok(NULL,sep);
	while ( cp == NULL ){
		printf(">");
		cp = fgets(ibuf,MXBUF,fi);
		if( cp != NULL ) cp = strtok(ibuf, sep);
	}
	cp = strupr(cp);   		// convert to uppercase
// printf("%s ",cp);
	return cp;
}

// show the first n registers
PRIVATE void showR(int n){
	int i, k;

	for( i = 0; i < n; i++ ){
		printf("R[%d] ",i);
		for( k = 0; k < NPE; k++)
			printf("%d ",R[k][i]);
		printf("\n");
	}
}

PRIVATE void dumpInternal(void){
	int  k;

	if( display ){
		showR(8);
		printf("LDS  ");
		for( k = 0; k < NPE; k++ )
			printf("%d ",LDS[k]);
		printf("\n");
	}
}

PRIVATE void trace(void){
	display = 0;
	while( !stopf && PC != brkp ) {
		execute();
		if( icnt > XLIMIT ){
			printf("infinite loop\n");
			break;
		}
	}
	display = 1;
}

PRIVATE int isReg(char *s){
	int n;
	if( s[0] == 'R') {
		n = atoi(s+1);
		if( n > 0 && n < 32 ) return 1;
	}
	return 0;
}

PRIVATE void setArg(char *s, int value){
	int k, n;
	if( isReg(s) ){				// set rX of all PEs
		n = atoi(s+1);
		for( k = 0; k < NPE; k++ )
			R[k][n] = value;
	}else if ( eqs(s,"PC") )
		PC = value;
	else if ( s[0] == 'M' )
		M[atoi(s+1)] = value;
	else
		printf("set unknow argument\n");
}

PRIVATE void dumpMem(int ads, int n){
	int i,k;
	k = 0;
	printf("%4d : ",ads);
	for(i=ads; i<ads+n; i++){
		printf("%d ",M[i]);
		k++;
		if( k%10 == 0 ) printf("\n%4d : ",i+1);
	}
	printf("\n");
}

PRIVATE void help(void){
	printf("%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n",
	  "g - go", "t - single step",
	  "b ads - set breakpoint", "s [rn,mn,pc] v - set",
	  "d ads n - dump", "r - show register",
	  "q - quit", "h - help");
}

void interp(void){
	char *arg;
	int i, v, v2;

	fi = stdin;
	while(runflag){
		arg = tok();
		switch( arg[0] ){
//		case 'A': 			// show all cores
//			disp = 1;
//			printf("mode show all cores\n");
//			break;
		case 'G': trace(); break;
		case 'T': execute(); dumpInternal(); break;
		case 'B': brkp = atoi(tok()); break;
//		case 'C':
//			v = atoi(tok());
//			if( v > NC ) v = NC-1;
//			currentcore = v;
//			disp = 0;
//			break;
		case 'S':
			arg = tok();
			v = atoi(tok());
			setArg(arg,v);
			break;
		case 'D':
			v = atoi(tok());
			v2 = atoi(tok());
			dumpMem(v,v2);
			break;
		case 'R': showR(32); break;
		case 'Q': runflag = 0; break;
		case 'H': help(); break;
		default: printf("unknow command\n");
		}
	}
}

/*
void testtok(){
	char *w;
	fi = stdin;
	cp = NULL;
	while(1){
		w = tok();
		printf("%s ",w);
		if( eqs(w,"QUIT")) break;
	}
}
*/
